@@ -731,7 +731,8 @@ query_join_add_cond_fields (QueryJoin *qj, QueryField *field1, QueryField *field
 		QueryView *qv = query_field_field_get_queryview (field1);
 		if ((qv != qj->priv->ant_view) && (qv != qj->priv->suc_view)) {
 			error = TRUE;
-			g_warning ("query_join_add_cond_fields: QueryView for field1 is not in join's views\n");
+			g_warning ("query_join_add_cond_fields: "
+				   "QueryView for field1 is not in join's views\n");
 		}
 	}
 
@@ -739,7 +740,8 @@ query_join_add_cond_fields (QueryJoin *qj, QueryField *field1, QueryField *field
 		QueryView *qv = query_field_field_get_queryview (field2);
 		if ((qv != qj->priv->ant_view) && (qv != qj->priv->suc_view)) {
 			error = TRUE;
-			g_warning ("query_join_add_cond_fields: QueryView for field2 is not in join's views\n");
+			g_warning ("query_join_add_cond_fields: "
+				   "QueryView for field2 is not in join's views\n");
 		}
 	}
 
@@ -770,7 +772,8 @@ query_join_add_cond (QueryJoin *qj, QueryCond *cond)
 			QueryView *qv = query_field_field_get_queryview (qf);
 			if ((qv != qj->priv->ant_view) && (qv != qj->priv->suc_view)) {
 				error = TRUE;
-				g_warning ("query_join_add_cond: QueryView for QF in QCond is not in join's views\n");
+				g_warning ("query_join_add_cond: "
+					   "QueryView for QF in QCond is not in join's views\n");
 			}
 		}
 		list = g_slist_next (list);
@@ -1000,7 +1003,7 @@ query_join_unset_skip_view (QueryJoin *qj)
 {
 	g_return_if_fail (qj && IS_QUERY_JOIN (qj));
 
-	qj->priv->skip_view = TRUE;
+	qj->priv->skip_view = FALSE;
 }
 
 gboolean
