@@ -143,53 +143,66 @@ server_data_type_update_list (ServerAccess * srv)
 					GDA_CONNECTION_SCHEMA_TYPES, NULL);
 
 	if (rs) {
-		total = gda_data_model_get_n_rows (rs);
-		now = 0;		
-		while (now < total) {
-			const GdaValue *value;
-
-			value = gda_data_model_get_value_at (rs, 0, now);
-			str = gda_value_stringify (value);
-			dt = server_data_type_get_from_name (dtl, str);
-			if (!dt) {
-				gchar *str2;
-
-				/* type name */
-				dt = SERVER_DATA_TYPE (server_data_type_new ());
-				server_data_type_set_sqlname (dt, str);
-
-				/* FIXME: number of params */
-				dt->numparams = 0;
-
-				/* description */
-				value = gda_data_model_get_value_at (rs, 2, now);
-				str2 = gda_value_stringify (value);
-				server_data_type_set_descr (dt, str2);
-				g_free (str2);
-				srv->data_types = g_slist_append (srv->data_types, dt);
+		if (server_resultset_check_data_model (rs, 4, 
+						       GDA_VALUE_TYPE_STRING, 
+						       GDA_VALUE_TYPE_STRING,
+						       GDA_VALUE_TYPE_STRING,
+						       GDA_VALUE_TYPE_TYPE)) {
+			total = gda_data_model_get_n_rows (rs);
+			now = 0;		
+			while (now < total) {
+				const GdaValue *value;
+				
+				value = gda_data_model_get_value_at (rs, 0, now);
+				str = gda_value_stringify (value);
+				dt = server_data_type_get_from_name (dtl, str);
+				if (!dt) {
+					gchar *str2;
+					
+					/* type name */
+					dt = SERVER_DATA_TYPE (server_data_type_new ());
+					server_data_type_set_sqlname (dt, str);
+					
+					/* FIXME: number of params */
+					dt->numparams = 0;
+					
+					/* description */
+					value = gda_data_model_get_value_at (rs, 2, now);
+					str2 = gda_value_stringify (value);
+					server_data_type_set_descr (dt, str2);
+					g_free (str2);
+					srv->data_types = g_slist_append (srv->data_types, dt);
 #ifdef debug_signal
-				g_print (">> 'DATA_TYPE_ADDED' from server_data_type_update_list\n");
+					g_print (">> 'DATA_TYPE_ADDED' from server_data_type_update_list\n");
 #endif
-				g_signal_emit_by_name (G_OBJECT (srv), "data_type_added", dt);
+					g_signal_emit_by_name (G_OBJECT (srv), "data_type_added", dt);
 #ifdef debug_signal
-				g_print ("<< 'DATA_TYPE_ADDED' from server_data_type_update_list\n");
+					g_print ("<< 'DATA_TYPE_ADDED' from server_data_type_update_list\n");
 #endif
+				}
+				g_free (str);
+				
+				dt->updated = TRUE;
+				
+				/* gda_type */
+				value = gda_data_model_get_value_at (rs, 3, now);
+				if (value && !gda_value_is_null (value)) 
+					dt->gda_type = gda_value_get_vtype (value);
+				
+				/* functions to display the data type */
+				dt->data_handler = server_access_get_handler_from_type (srv, dt);
+				
+				g_signal_emit_by_name (G_OBJECT (srv), "progress", "DATA_TYPES", now, total);
+				now++;
 			}
-			g_free (str);
-
-			dt->updated = TRUE;
-			
-			/* gda_type */
-			value = gda_data_model_get_value_at (rs, 3, now);
-			if (value && !gda_value_is_null (value)) 
-				dt->gda_type = gda_value_get_vtype (value);
-
-			/* functions to display the data type */
-			dt->data_handler = server_access_get_handler_from_type (srv, dt);
-
-			g_signal_emit_by_name (G_OBJECT (srv), "progress", "DATA_TYPES", now, total);
-			now++;
 		}
+		else
+			g_warning ("The provider has returned the wrong \n"
+				   "number and/or types of data\n"
+				   "for the TYPES schema description.\n"
+				   "Please contact the provider's maintainer "
+				   "and report the problem.");
+
 		g_object_unref (G_OBJECT (rs));
 
 		/* remove the data types not existing anymore */
@@ -448,145 +461,161 @@ server_function_update_list (ServerAccess * srv)
 	GSList *list, *hlist, *hold;
 	gint i;
 	gboolean insert, found;
-	ServerDataType *dt;
 	ServerFunction *df = NULL;
 	gchar *str;
-	GdaDataModel *rs, *rs2;
+	GdaDataModel *rs;
 	guint now, total;
 
 	rs = gda_connection_get_schema (GDA_CONNECTION (srv->cnc),
 					GDA_CONNECTION_SCHEMA_PROCEDURES, NULL); 
 	
 	if (rs) {
-		total = gda_data_model_get_n_rows (rs);
-		now = 0;
-		while (now < total) {
-			ServerDataType *rettype = NULL;	/* return type for the function */
-			GSList *dtl = NULL;	/* list of params for the function */
-			const GdaValue *value;
-			gchar *ptr;
-
-			/* do we have to insert that function in the list ? */
-			value = gda_data_model_get_value_at (rs, 1, now);
-			str = gda_value_stringify (value);
-			df = server_function_get_from_objid (srv->data_functions, str);
-			g_free (str);
-
-			/* build the parameters list for the function */
-			insert = TRUE;
-			
-			/* return type */
-			value = gda_data_model_get_value_at (rs, 4, now);
-			str = gda_value_stringify (value);
-			rettype = server_data_type_get_from_name (srv->data_types, str);
-			g_free (str);
-			if (!rettype)
-				insert = FALSE;
-
-			/* argument types */
-			value = gda_data_model_get_value_at (rs, 6, now);
-			str = gda_value_stringify (value);
-			ptr = strtok (str, " ");
-			while (ptr && *ptr) {
-				ServerDataType *indt;
-
-				if (*ptr == '-') 
-					dtl = g_slist_append (dtl, NULL); /* any data type will do */
-				else {
-					indt = server_data_type_get_from_name (srv->data_types, ptr);
-					if (indt)
-						dtl = g_slist_append (dtl, indt);
-					else
-						insert = FALSE;
-				}
-				ptr = strtok (NULL, " ");
-			}
-			g_free (str);
-
-
-			/* does the function we found have the same rettype and params 
-			   as the one we have now? */
-			if (insert && df) {
-				gboolean isequal = TRUE;
-				list = df->args;
-				hlist = dtl;
-				while (list && hlist && isequal) {
-					if (list->data != hlist->data)
-						isequal = FALSE;
-					list = g_slist_next (list);
-					hlist = g_slist_next (hlist);
-				}
-				if (isequal && (df->result_type != rettype))
-					isequal = FALSE;
-				insert = !isequal;	/* insert only if different */
-				if (isequal)
-					df->updated = TRUE;
-			}
-
-
-			/* actual creation step */
-			if (insert) {
-				/* creating new ServerFunction object */
-				df = SERVER_FUNCTION (server_function_new ());
-				df->result_type = rettype;
-				df->args = dtl;
-				value = gda_data_model_get_value_at (rs, 3, now);
-				if (* gda_value_get_string(value)) {
-					str = gda_value_stringify (value);
-					server_function_set_descr (df, str);
-					g_free (str);
-				}
-				else
-					server_function_set_descr (df, NULL);
-
-				value =  gda_data_model_get_value_at (rs, 0, now);
-				str = gda_value_stringify (value);
-				server_function_set_sqlname (df, str);
-				g_free (str);
+		if (server_resultset_check_data_model (rs, 8, 
+						       GDA_VALUE_TYPE_STRING, 
+						       GDA_VALUE_TYPE_STRING,
+						       GDA_VALUE_TYPE_STRING,
+						       GDA_VALUE_TYPE_STRING,
+						       GDA_VALUE_TYPE_STRING,
+						       GDA_VALUE_TYPE_INTEGER,
+						       GDA_VALUE_TYPE_STRING,
+						       GDA_VALUE_TYPE_STRING)) {
+			total = gda_data_model_get_n_rows (rs);
+			now = 0;
+			while (now < total) {
+				ServerDataType *rettype = NULL;	/* return type for the function */
+				GSList *dtl = NULL;	/* list of params for the function */
+				const GdaValue *value;
+				gchar *ptr;
 
+				/* do we have to insert that function in the list ? */
 				value = gda_data_model_get_value_at (rs, 1, now);
 				str = gda_value_stringify (value);
-				df->objectid = str;
-
-				value = gda_data_model_get_value_at (rs, 2, now);
-				str = gda_value_stringify (value);
-				if (strcmp (str, srv->user_name->str))
-					df->is_user = FALSE;
-				else
-					df->is_user = TRUE;
+				df = server_function_get_from_objid (srv->data_functions, str);
 				g_free (str);
-
-
-				/* insertion in the list */
-				/* finding where to insert the function */
-				list = srv->data_functions;
-				i = 0;
-				found = FALSE;
-				while (list && !found) {
-					if (strcmp (df->sqlname, SERVER_FUNCTION (list->data)->sqlname) < 0)
-						found = TRUE;
-					else
-						i++;
-					list = g_slist_next (list);
+				
+				/* build the parameters list for the function */
+				insert = TRUE;
+				
+				/* return type */
+				value = gda_data_model_get_value_at (rs, 4, now);
+				str = gda_value_stringify (value);
+				rettype = server_data_type_get_from_name (srv->data_types, str);
+				g_free (str);
+				if (!rettype)
+					insert = FALSE;
+				
+				/* argument types */
+				value = gda_data_model_get_value_at (rs, 6, now);
+				str = gda_value_stringify (value);
+				ptr = strtok (str, " ");
+				while (ptr && *ptr) {
+					ServerDataType *indt;
+					
+					if (*ptr == '-') 
+						dtl = g_slist_append (dtl, NULL); /* any data type will do */
+					else {
+						indt = server_data_type_get_from_name (srv->data_types, ptr);
+						if (indt)
+							dtl = g_slist_append (dtl, indt);
+						else
+							insert = FALSE;
+					}
+					ptr = strtok (NULL, " ");
 				}
-				srv->data_functions = g_slist_insert (srv->data_functions, df, i);
+				g_free (str);
+				
+				
+				/* does the function we found have the same rettype and params 
+				   as the one we have now? */
+				if (insert && df) {
+					gboolean isequal = TRUE;
+					list = df->args;
+					hlist = dtl;
+					while (list && hlist && isequal) {
+						if (list->data != hlist->data)
+							isequal = FALSE;
+						list = g_slist_next (list);
+						hlist = g_slist_next (hlist);
+					}
+					if (isequal && (df->result_type != rettype))
+						isequal = FALSE;
+					insert = !isequal;	/* insert only if different */
+					if (isequal)
+						df->updated = TRUE;
+				}
+				
+				
+				/* actual creation step */
+				if (insert) {
+					/* creating new ServerFunction object */
+					df = SERVER_FUNCTION (server_function_new ());
+					df->result_type = rettype;
+					df->args = dtl;
+					value = gda_data_model_get_value_at (rs, 3, now);
+					if (* gda_value_get_string(value)) {
+						str = gda_value_stringify (value);
+						server_function_set_descr (df, str);
+						g_free (str);
+					}
+					else
+						server_function_set_descr (df, NULL);
+					
+					value =  gda_data_model_get_value_at (rs, 0, now);
+					str = gda_value_stringify (value);
+					server_function_set_sqlname (df, str);
+					g_free (str);
+					
+					value = gda_data_model_get_value_at (rs, 1, now);
+					str = gda_value_stringify (value);
+					df->objectid = str;
+					
+					value = gda_data_model_get_value_at (rs, 2, now);
+					str = gda_value_stringify (value);
+					if (strcmp (str, srv->user_name->str))
+						df->is_user = FALSE;
+					else
+						df->is_user = TRUE;
+					g_free (str);
+					
+					
+					/* insertion in the list */
+					/* finding where to insert the function */
+					list = srv->data_functions;
+					i = 0;
+					found = FALSE;
+					while (list && !found) {
+						if (strcmp (df->sqlname, SERVER_FUNCTION (list->data)->sqlname) < 0)
+							found = TRUE;
+						else
+							i++;
+						list = g_slist_next (list);
+					}
+					srv->data_functions = g_slist_insert (srv->data_functions, df, i);
 #ifdef debug_signal
-				g_print (">> 'DATA_FUNCTION_ADDED' from server_function_update_list\n");
+					g_print (">> 'DATA_FUNCTION_ADDED' from server_function_update_list\n");
 #endif
-				g_signal_emit_by_name (G_OBJECT (srv), "data_function_added", df);
+					g_signal_emit_by_name (G_OBJECT (srv), "data_function_added", df);
 #ifdef debug_signal
-				g_print ("<< 'DATA_FUNCTION_ADDED' from server_function_update_list\n");
+					g_print ("<< 'DATA_FUNCTION_ADDED' from server_function_update_list\n");
 #endif
-				df->updated = TRUE;
+					df->updated = TRUE;
+				}
+				else 	/* we do not insert the function, so free dtl */
+					g_slist_free (dtl);
+				
+				g_signal_emit_by_name (G_OBJECT (srv), "progress",
+						       "FUNCTIONS",
+						       now, total);
+				now ++;
 			}
-			else 	/* we do not insert the function, so free dtl */
-				g_slist_free (dtl);
-
-			g_signal_emit_by_name (G_OBJECT (srv), "progress",
-						 "FUNCTIONS",
-						 now, total);
-			now ++;
 		}
+		else
+			g_warning ("The provider has returned the wrong \n"
+				   "number and/or types of data\n"
+				   "for the PROCEDURES schema description.\n"
+				   "Please contact the provider's maintainer "
+				   "and report the problem.");
+
 		g_object_unref (G_OBJECT (rs));
 	}
 
@@ -900,79 +929,95 @@ server_aggregate_update_list (ServerAccess * srv)
 	rs = gda_connection_get_schema (GDA_CONNECTION (srv->cnc),
 					GDA_CONNECTION_SCHEMA_AGGREGATES, NULL);
 	if (rs) {
-		total = gda_data_model_get_n_rows (rs);
-		now = 0;
-		while (now < total) {
-			/* in type */
-			value = gda_data_model_get_value_at (rs, 5, now);
-			if (!gda_value_is_null (value)) {
-				str = gda_value_stringify (value);
-				dt = server_data_type_get_from_name (srv->data_types, str);
-				g_free (str);
-			}
-			else
-				dt = NULL;
-
-			/* aggregate id */
-			value = gda_data_model_get_value_at (rs, 1, now);	
-			str = gda_value_stringify (value);
-			da = server_aggregate_get_from_objid (srv->data_aggregates, str);
-			g_free (str);
-
-			if ((da && (da->arg_type != dt)) || (!da)) {
-				/* create a new DataAggregate */
-				da = SERVER_AGGREGATE (server_aggregate_new ());
-				da->arg_type = dt;
-
-				/* sql name */
-				value = gda_data_model_get_value_at (rs, 0, now);
-				str = gda_value_stringify (value);
-				da->sqlname = str;
-
-				/* Id */
-				value = gda_data_model_get_value_at (rs, 1, now);
-				str = gda_value_stringify (value);
-				da->objectid = str;
-
-
-				/* comments */
-				value = gda_data_model_get_value_at (rs, 3, now);
+		if (server_resultset_check_data_model (rs, 7, 
+						       GDA_VALUE_TYPE_STRING, 
+						       GDA_VALUE_TYPE_STRING,
+						       GDA_VALUE_TYPE_STRING,
+						       GDA_VALUE_TYPE_STRING,
+						       GDA_VALUE_TYPE_STRING,
+						       GDA_VALUE_TYPE_STRING,
+						       GDA_VALUE_TYPE_STRING)) {
+			total = gda_data_model_get_n_rows (rs);
+			now = 0;
+			while (now < total) {
+				/* in type */
+				value = gda_data_model_get_value_at (rs, 5, now);
 				if (!gda_value_is_null (value)) {
 					str = gda_value_stringify (value);
-					da->descr = str;
+					dt = server_data_type_get_from_name (srv->data_types, str);
+					g_free (str);
 				}
+				else
+					dt = NULL;
+				
+				/* aggregate id */
+				value = gda_data_model_get_value_at (rs, 1, now);	
+				str = gda_value_stringify (value);
+				da = server_aggregate_get_from_objid (srv->data_aggregates, str);
+				g_free (str);
+				
+				if ((da && (da->arg_type != dt)) || (!da)) {
+					/* create a new DataAggregate */
+					da = SERVER_AGGREGATE (server_aggregate_new ());
+					da->arg_type = dt;
+					
+					/* sql name */
+					value = gda_data_model_get_value_at (rs, 0, now);
+					str = gda_value_stringify (value);
+					da->sqlname = str;
+					
+					/* Id */
+					value = gda_data_model_get_value_at (rs, 1, now);
+					str = gda_value_stringify (value);
+					da->objectid = str;
 
-				/* insertion into the list */
-				j = 0;
-				found = FALSE;
-				list = srv->data_aggregates;
-				while (list && !found) {
-					if (strcmp (da->sqlname, SERVER_AGGREGATE (list->data)->sqlname) > 0)
-						j++;
-					else
-						found = TRUE;
-					list = g_slist_next (list);
-				}
-
-				srv->data_aggregates = g_slist_insert (srv->data_aggregates, da, j);
+					/* comments */
+					value = gda_data_model_get_value_at (rs, 3, now);
+					if (!gda_value_is_null (value)) {
+						str = gda_value_stringify (value);
+						da->descr = str;
+					}
+					
+					/* insertion into the list */
+					j = 0;
+					found = FALSE;
+					list = srv->data_aggregates;
+					while (list && !found) {
+						if (strcmp (da->sqlname, SERVER_AGGREGATE (list->data)->sqlname) > 0)
+							j++;
+						else {
+							found = TRUE;
+							break;
+						}
+						list = g_slist_next (list);
+					}
+					
+					srv->data_aggregates = g_slist_insert (srv->data_aggregates, da, j);
 #ifdef debug_signal
-				g_print (">> 'DATA_AGGREGATE_ADDED' from "
-					 "server_aggregate_update_list\n");
+					g_print (">> 'DATA_AGGREGATE_ADDED' from "
+						 "server_aggregate_update_list\n");
 #endif
-				g_signal_emit_by_name (G_OBJECT (srv), "data_aggregate_added", da);
+					g_signal_emit_by_name (G_OBJECT (srv), "data_aggregate_added", da);
 #ifdef debug_signal
-				g_print ("<< 'DATA_AGGREGATE_ADDED' from "
-					 "server_aggregate_update_list\n");
+					g_print ("<< 'DATA_AGGREGATE_ADDED' from "
+						 "server_aggregate_update_list\n");
 #endif
-				da->updated = TRUE;
-			}
-			else {
-				if (da && (da->arg_type == dt))	/* mark as updated */
 					da->updated = TRUE;
+				}
+				else {
+					if (da && (da->arg_type == dt))	/* mark as updated */
+						da->updated = TRUE;
+				}
+				g_signal_emit_by_name (G_OBJECT (srv), "progress", "AGGREGATES", now, total);
+				now++;
 			}
-			g_signal_emit_by_name (G_OBJECT (srv), "progress", "AGGREGATES", now, total);
-			now++;
 		}
+		else
+			g_warning ("The provider has returned the wrong \n"
+				   "number and/or types of data\n"
+				   "for the AGGREGATES schema description.\n"
+				   "Please contact the provider's maintainer "
+				   "and report the problem.");
 		g_object_unref (G_OBJECT (rs));
 	}
 
